package com.mortals.xhx.feign.area;

import com.alibaba.fastjson.JSON;
import com.mortals.xhx.common.code.ApiRespCodeEnum;
import com.mortals.xhx.common.pdu.SitePdu;
import com.mortals.xhx.feign.IFeign;
import com.mortals.xhx.module.area.model.AreaEntity;
import com.mortals.xhx.feign.rsp.ApiResp;
import feign.hystrix.FallbackFactory;
import lombok.extern.slf4j.Slf4j;
import org.springframework.cloud.openfeign.FeignClient;
import org.springframework.stereotype.Component;
import org.springframework.web.bind.annotation.*;

/**
 * 区域
 *
 * @author: zxfei
 * @date: 2022/5/30 10:40
 */
@FeignClient(name = "base-manager", path = "/base", fallback = AreaFeignFallbackFactory.class)
public interface IApiAreaFeign extends IFeign {


    /**
     * 查看区域列表信息
     *
     * @param query
     * @return
     */
    @PostMapping(value = "/area/list")
    String list(@RequestBody AreaEntity query);


    /**
     * 查看区域信息
     *
     * @param id
     * @return
     */
    @GetMapping(value = "/area/info")
    String viewAreaInfo(@RequestParam(value = "id") Long id);


    /**
     * 查看区域子信息
     *
     * @param parentId 0为顶级
     * @return
     */
    @GetMapping(value = "/area/getListByParentId")
    String getListByParentId(@RequestParam(value = "parentId") String parentId);


    /**
     * 查看区域列表信息
     *
     * @return
     */
    @PostMapping(value = "/area/treeselect")
    String treeselect();


    /**
     * 根据区域id查询站点列表ids
     *
     * @param query
     * @return
     */
    @PostMapping(value = "/site/getFlatSitesByAreaIds")
    String getFlatSitesByAreaIds(@RequestBody SitePdu query);

    /**
     * 根据条件查询站点列表
     * @param query
     * @return
     */
    @PostMapping(value = "/site/list")
    String getSitesByQuery(@RequestBody SitePdu query);
}


@Slf4j
@Component
class AreaFeignFallbackFactory implements FallbackFactory<IApiAreaFeign> {
    @Override
    public IApiAreaFeign create(Throwable t) {
        return new IApiAreaFeign() {
            @Override
            public String list(AreaEntity query) {
                log.error("暂时无法获取区域列表信息，请稍后再试!query:{}", JSON.toJSONString(query));
                log.error(t.getMessage());
                ApiResp<String> failResp = new ApiResp<>();
                failResp.setCode(ApiRespCodeEnum.FAILED.getValue());
                failResp.setMsg("暂时无法获取区域列表信息，请稍后再试!");
                return JSON.toJSONString(failResp);
            }

            @Override
            public String viewAreaInfo(Long id) {
                log.error("暂时无法获取区域信息，请稍后再试!parentId:{}", id);
                log.error(t.getMessage());
                ApiResp<String> failResp = new ApiResp<>();
                failResp.setCode(ApiRespCodeEnum.FAILED.getValue());
                failResp.setMsg("暂时无法获取区域信息，请稍后再试!");
                return JSON.toJSONString(failResp);
            }

            @Override
            public String getListByParentId(String parentId) {
                log.error("暂时无法获取区域信息，请稍后再试!parentId:{}", parentId);
                log.error(t.getMessage());
                ApiResp<String> failResp = new ApiResp<>();
                failResp.setCode(ApiRespCodeEnum.FAILED.getValue());
                failResp.setMsg("暂时无法获取区域信息，请稍后再试!");
                return JSON.toJSONString(failResp);
            }

            @Override
            public String treeselect() {
                ApiResp<String> failResp = new ApiResp<>();
                failResp.setCode(ApiRespCodeEnum.FAILED.getValue());
                failResp.setMsg("暂时无法获取区域信息，请稍后再试!");
                return JSON.toJSONString(failResp);
            }

            @Override
            public String getFlatSitesByAreaIds(SitePdu query) {
                ApiResp<String> failResp = new ApiResp<>();
                failResp.setCode(ApiRespCodeEnum.FAILED.getValue());
                failResp.setMsg("暂时无法获取站点列表，请稍后再试!");
                return JSON.toJSONString(failResp);
            }

            @Override
            public String getSitesByQuery(SitePdu query) {
                ApiResp<String> failResp = new ApiResp<>();
                failResp.setCode(ApiRespCodeEnum.FAILED.getValue());
                failResp.setMsg("暂时无法获取站点列表，请稍后再试!");
                return JSON.toJSONString(failResp);
            }
        };
    }
}

