package com.mortals.xhx.feign.user;

import com.alibaba.fastjson.JSON;
import com.mortals.xhx.common.code.ApiRespCodeEnum;
import com.mortals.xhx.feign.IFeign;
import com.mortals.xhx.feign.req.BaseUserQuery;
import com.mortals.xhx.feign.req.BaseUserReq;
import com.mortals.xhx.feign.rsp.ApiResp;
import com.mortals.xhx.feign.rsp.UserRsp;
import feign.hystrix.FallbackFactory;
import lombok.extern.slf4j.Slf4j;
import org.springframework.cloud.openfeign.FeignClient;
import org.springframework.stereotype.Component;
import org.springframework.web.bind.annotation.PostMapping;
import org.springframework.web.bind.annotation.RequestBody;

/**
 * 用户服务
 *
 * @author: zxfei
 * @date: 2022/5/30 10:40
 */
@FeignClient(name = "portal-manager", path = "/zwfw", fallback = UserFeignFallbackFactory.class)
public interface IApiUserFeign extends IFeign {

    /**
     * 查询用户列表
     *
     * @param req
     * @return
     */
    @PostMapping("/api/user/findAllUser")
    ApiResp<UserRsp> findAllUser(@RequestBody BaseUserReq<BaseUserQuery> req);

}


@Slf4j
@Component
class UserFeignFallbackFactory implements FallbackFactory<IApiUserFeign> {
    @Override
    public IApiUserFeign create(Throwable t) {
        return new IApiUserFeign() {
            @Override
            public ApiResp<UserRsp> findAllUser(BaseUserReq<BaseUserQuery> req) {
                ApiResp<UserRsp> failResp = new ApiResp<>();
                failResp.setCode(ApiRespCodeEnum.FAILED.getValue());
                failResp.setMsg("暂时无法获取站点列表，请稍后再试!");
                return failResp;
            }
        };
    }
}