package com.mortals.xhx.module.user.web;

import com.mortals.framework.annotation.UnAuth;
import com.mortals.framework.common.Rest;
import com.mortals.framework.exception.AppException;
import com.mortals.framework.model.PageInfo;
import com.mortals.framework.model.Result;
import com.mortals.framework.web.BaseCRUDJsonBodyMappingController;
import com.mortals.xhx.base.framework.annotation.Operlog;
import com.mortals.xhx.module.area.model.AreaQuery;
import com.mortals.xhx.module.param.service.ParamService;
import com.mortals.xhx.module.role.model.RoleEntity;
import com.mortals.xhx.module.role.model.RoleQuery;
import com.mortals.xhx.module.role.model.RoleUserEntity;
import com.mortals.xhx.module.role.model.RoleUserQuery;
import com.mortals.xhx.module.role.service.RoleService;
import com.mortals.xhx.module.role.service.RoleUserService;
import com.mortals.xhx.module.user.model.UserEntityExt;
import io.swagger.annotations.Api;
import io.swagger.annotations.ApiOperation;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.util.ObjectUtils;
import org.springframework.web.bind.annotation.RequestMapping;
import org.springframework.web.bind.annotation.RestController;
import org.springframework.web.bind.annotation.*;
import com.mortals.framework.model.Context;

import javax.servlet.http.HttpServletRequest;
import javax.servlet.http.HttpServletResponse;

import com.mortals.framework.web.BaseCRUDJsonBodyMappingController;
import com.mortals.xhx.module.user.model.UserEntity;
import com.mortals.xhx.module.user.model.UserQuery;
import com.mortals.xhx.module.user.service.UserService;
import org.apache.commons.lang3.ArrayUtils;
import com.mortals.framework.util.StringUtils;

import java.util.HashMap;
import java.util.List;
import java.util.Map;
import java.util.stream.Collectors;

import com.alibaba.fastjson.JSONObject;

import java.util.Arrays;

import org.springframework.web.bind.annotation.*;
import org.springframework.web.multipart.MultipartFile;

import static com.mortals.framework.ap.SysConstains.*;

/**
 * 用户信息业务
 *
 * @author zxfei
 * @date 2022-05-25
 */
@Api(value = "用户信息控制器", tags = {"用户信息管理"})

@RestController
@RequestMapping("user")
public class UserController extends BaseCRUDJsonBodyMappingController<UserService, UserEntity, Long> {

    @Autowired
    private ParamService paramService;

    @Autowired
    private RoleUserService roleUserService;
    @Autowired
    private RoleService roleService;

    public UserController() {
        super.setModuleDesc("用户信息业务");
    }

    @Override
    protected void init(Map<String, Object> model, Context context) {
        this.addDict(model, "userType", paramService.getParamBySecondOrganize("User", "userType"));
        this.addDict(model, "status", paramService.getParamBySecondOrganize("User", "status"));
        List<RoleEntity> roleList = roleService.find(new RoleEntity());
        super.addDict(model, "roleId", roleList.stream().collect(Collectors.toMap(e->e.getId().toString(), e->e.getName())));
        super.init(model, context);
    }


    @Override
    protected int viewAfter(Long id, Map<String, Object> model, UserEntity entity, Context context) throws AppException {
        Map<Long, String> roleMap = roleService.find(new RoleQuery()).stream().collect(Collectors.toMap(x -> x.getId(), y -> y.getName()));
        List<RoleUserEntity> roleUserList = roleUserService.find(new RoleUserQuery().userId(id));
        if(!ObjectUtils.isEmpty(roleUserList)){
            entity.setRoleId(roleUserList.stream().map(RoleUserEntity::getRoleId).map(String::valueOf).collect(Collectors.joining(",")));
            entity.setRoleName(roleUserList.stream().map(m->roleMap.get(m.getRoleId())).filter(f->f!=null).collect(Collectors.joining(",")));

        }
        return super.viewAfter(id, model, entity, context);
    }

    @Override
    public Rest<Object> list(UserEntity query) {
        Map<String, Object> model = new HashMap();
        Rest<Object> ret = new Rest<>();
        Context context = this.getContext();
        String busiDesc = "查询" + this.getModuleDesc();
        int code;
        try {
            PageInfo pageInfo = this.buildPageInfo(query);
            this.doListBefore(query, model, context);
            Result result = this.getService().findExt(query, pageInfo, context);
            model.put("data", result.getList());
            model.put("pageInfo", result.getPageInfo());
            super.parsePageInfo(model,result.getPageInfo());
            this.parsePageInfo(model, result.getPageInfo());
            code = this.doListAfter(query, model, context);
            this.recordSysLog(this.request, busiDesc + " 【成功】");
        } catch (Exception var9) {
            code = -1;
            this.doException(this.request, busiDesc, model, var9);
        }

        this.init(model, context);
        ret.setCode(code);
        ret.setData(model);
        ret.setDict(model.get(KEY_RESULT_DICT));
        ret.setMsg(model.get(MESSAGE_INFO)==null?"":model.remove(MESSAGE_INFO).toString());
        return ret;
    }

    /* @Override
    public String list(@RequestBody(required = false) UserEntity query) {
        Map<String, Object> model = new HashMap();
        JSONObject ret = new JSONObject();
        Context context = this.getContext();
        String busiDesc = "查询" + this.getModuleDesc();
        int code;
        try {
            PageInfo pageInfo = this.buildPageInfo(query);
            this.doListBefore(query, model, context);
            Result result = this.getService().findExt(query, pageInfo, context);
            model.put("data", result.getList());
            model.put("pageInfo", result.getPageInfo());
            this.parsePageInfo(model, result.getPageInfo());
            code = this.doListAfter(query, model, context);
            this.recordSysLog(this.request, busiDesc + " 【成功】");
        } catch (Exception var9) {
            code = -1;
            this.doException(this.request, busiDesc, model, var9);
        }

        this.init(model, context);
        ret.put("code", code);
        ret.put("msg", model.remove("message_info"));
        ret.put("dict", model.remove("dict"));
        ret.put("data", model);
        return ret.toJSONString();
    }*/


    @RequestMapping(value = "change/password", method = RequestMethod.POST)
    public String changePassword(@RequestParam String oldPwd, @RequestParam String newPwd) {
        JSONObject ret = new JSONObject();
        try {
            service.updateUserPwd(super.getCurUser().getLoginName(), oldPwd, newPwd);
            ret.put(KEY_RESULT_CODE, VALUE_RESULT_SUCCESS);
            ret.put(KEY_RESULT_MSG, "密码修改成功！");
        } catch (Exception e) {
            ret.put(KEY_RESULT_CODE, VALUE_RESULT_FAILURE);
            ret.put(KEY_RESULT_MSG, super.convertException(e));
        }
        return ret.toJSONString();
    }

    /**
     * 站点授权
     * @return
     */
    @PostMapping(value = "siteAuth")
    public String siteAuth(@RequestBody UserEntityExt entityExt) {
        JSONObject ret = new JSONObject();
        Map<String, Object> model = new HashMap<>();
        String busiDesc =  this.getModuleDesc() + "站点授权";
        try {
            this.service.siteAuth(entityExt,getContext());
            recordSysLog(request, busiDesc + "【成功】");
            ret.put(KEY_RESULT_CODE, VALUE_RESULT_SUCCESS);
            ret.put(KEY_RESULT_MSG, "授权成功！");
        } catch (Exception e) {
            log.error("查询站点信息错误", e);
            this.doException(request, busiDesc, model, e);
            ret.put(KEY_RESULT_CODE, VALUE_RESULT_FAILURE);
            ret.put(KEY_RESULT_MSG, e.getMessage());
        }
        return ret.toJSONString();
    }
}